﻿--[[
	Tipachu
		Adds item icons to tooltips
--]]

local DEFAULT_ICON_SIZE = 24

--add icon to the tooltip
local function setTooltipIcon(self, icon)
	local title = icon and _G[self:GetName() .. 'TextLeft1']
	if title then
		title:SetFormattedText('|T%s:%d|t %s', icon, _G['TipachuSize'] or DEFAULT_ICON_SIZE, title:GetText())
	end
end

--tooltip hooking generator
local function newTooltipHooker(method, func)
	return function(tooltip)
		local modified = false

		tooltip:HookScript('OnTooltipCleared', function(self, ...)
			modified = false
		end)

		tooltip:HookScript(method, function(self, ...)
			if not modified  then
				modified = true
				func(self, ...)
			end
		end)
	end
end

local hookItem = newTooltipHooker('OnTooltipSetItem', function(self, ...)
	local name, link = self:GetItem()
	if link then
		setTooltipIcon(self, GetItemIcon(link))
	end
end)

local hookSpell = newTooltipHooker('OnTooltipSetSpell', function(self, ...)
	local name, rank, id = self:GetSpell()
	if id then
		setTooltipIcon(self, select(3, GetSpellInfo(id)))
	end
end)

--hook tooltips
for _, tooltip in pairs{GameTooltip, ItemRefTooltip} do
	hookItem(tooltip)
	hookSpell(tooltip)
end

DEFAULT_CHAT_FRAME:AddMessage(" \n|cFF87CEEB欢迎使用散落于云海制作的Mirror UI，本插件包唯一发布地址为|cFFFF6100http://nga.178.com/read.php?tid=4325974&_fp=1&_ff=200|cFF87CEEB。\n任何将本插件包用于商业用途的行为都是|cFFFF6100未经本人许可|cFF87CEEB并|cFFFF6100违反本人意愿|cFF87CEEB的。\n欢迎去我的帖子里提交BUG或者建议，使用上有问题我也会为你们提供耐心的解答。");